@@ -16,6 +16,48 @@
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ *
+ * mozldap_ldap_explode, mozldap_ldap_explode_dn, mozldap_ldap_explode_rdn
+ * and ldap_utf8 functions
+ * are from the file ldap/libraries/libldap/getdn.c in the Mozilla LDAP C SDK
+ *
+ * Version: MPL 1.1/GPL 2.0/LGPL 2.1
+ *
+ * The contents of this file are subject to the Mozilla Public License Version
+ * 1.1 (the "License"); you may not use this file except in compliance with
+ * the License. You may obtain a copy of the License at
+ * http://www.mozilla.org/MPL/
+ *
+ * Software distributed under the License is distributed on an "AS IS" basis,
+ * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
+ * for the specific language governing rights and limitations under the
+ * License.
+ *
+ * The Original Code is Mozilla Communicator client code, released
+ * March 31, 1998.
+ *
+ * The Initial Developer of the Original Code is
+ * Netscape Communications Corporation.
+ * Portions created by the Initial Developer are Copyright (C) 1998-1999
+ * the Initial Developer. All Rights Reserved.
+ *
+ * Contributor(s):
+ *
+ * Alternatively, the contents of this file may be used under the terms of
+ * either of the GNU General Public License Version 2 or later (the "GPL"),
+ * or the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
+ * in which case the provisions of the GPL or the LGPL are applicable instead
+ * of those above. If you wish to allow use of your version of this file only
+ * under the terms of either the GPL or the LGPL, and not to allow others to
+ * use your version of this file under the terms of the MPL, indicate your
+ * decision by deleting the provisions above and replace them with the notice
+ * and other provisions required by the GPL or the LGPL. If you do not delete
+ * the provisions above, a recipient may use your version of this file under
+ * the terms of any one of the MPL, the GPL or the LGPL.
+ *
+ *  Copyright (c) 1994 Regents of the University of Michigan.
+ *  All rights reserved.
+ *
  * END COPYRIGHT BLOCK **/
 #include <stdio.h>
 #include <time.h>
@@ -37,20 +79,6 @@
 #define strcasecmp stricmp
 #define strncasecmp _strnicmp
 #endif
-
-#ifdef XP_UNIX
-/*
- * Version information for the 'ident' and 'what commands
- *
- * NOTE: the first component of the concatenated rcsid string
- * must not end in a '$' to prevent rcs keyword substitution.
- */
-static char rcsid[] = "$Header: AdminUtil " ADMSDK_PRODUCT  
-        "  " ADMSDK_BUILDNUM " $";
-static char sccsid[] = "@(#)AdminUtil " ADMSDK_PRODUCT  
-		"  " ADMSDK_BUILDNUM;
-#endif /* XP_UNIX */
-
  
 /* Max size for a pathname */
 #ifndef PATH_MAX
@@ -351,22 +379,36 @@ dn2AttrName(char* dn, char* rootDN)
 
   memset(buf, '\0', sizeof(buf));
 
-  dnList = ldap_explode_dn(dn, 1);
-  rootList = ldap_explode_dn(rootDN, 1);
+  dnList = admldap_ldap_explode_dn(dn, 1);
+  rootList = admldap_ldap_explode_dn(rootDN, 1);
 
-  while (rootList[rootLen]) rootLen++;
-  while (dnList[dnLen]) dnLen++;
+  if (rootList && dnList) {
+    while (rootList[rootLen]) rootLen++;
+    while (dnList[dnLen]) dnLen++;
 
-  attrLen = dnLen - rootLen;
+    attrLen = dnLen - rootLen;
 
-  while (attrLen > 0) {
-    if (attrLen == 1)
-      PR_snprintf(buf, sizeof(buf), "%s%s", buf, dnList[0]);
-    else
-      PR_snprintf(buf, sizeof(buf), "%s%s.", buf, dnList[attrLen-1]);
-    attrLen--;
+    while (attrLen > 0) {
+      if (attrLen == 1)
+        PR_snprintf(buf, sizeof(buf), "%s%s", buf, dnList[0]);
+      else
+        PR_snprintf(buf, sizeof(buf), "%s%s.", buf, dnList[attrLen-1]);
+      attrLen--;
+    }
+  } else {
+    char *envvar = PR_GetEnv("ADMINUTIL_DEBUG");
+    if (!dnList) {
+      if (envvar && (*envvar == '1')) {
+        fprintf(stderr, "dn2attrName: Error: invalid dn [%s]\n", dn);
+      }
+    }
+    if (!rootList) {
+      if (envvar && (*envvar == '1')) {
+        fprintf(stderr, "dn2attrName: Error: invalid rootDN [%s]\n", rootDN);
+      }
+    }
+    PL_strncpy(buf, "unknown", sizeof(buf));
   }
-
   admutil_strsFree(dnList);
   admutil_strsFree(rootList);
 
@@ -394,6 +436,11 @@ attrName2dn(char* attrName, char* rootDN)
       rdn = attrPtr+1;
     }
     else rdn = buf;
+    /* NOTE: FIXME https://fedorahosted.org/389/ticket/47546 */
+    /* This allows you to create bad DNs
+     ** no escaping for the rdn value
+     ** spaces after the comma
+     */
     if (result[0] == '\0')
       PR_snprintf(resultPtr, sizeof(result), "cn=%s", rdn);
     else
@@ -2286,17 +2333,19 @@ admutil_ldap_init(
 #if defined(USE_OPENLDAP)
       char *certdir = admldapGetSecurityDir(info);
       int optval = 0;
+#else
+      LDAP *myld = NULL;
 #endif /* !USE_OPENLDAP */
       int ssl_strength = 0;
-      LDAP *myld = NULL;
 
+#if !defined(USE_OPENLDAP)
       /* we can only use the set functions below with a real
          LDAP* if it has already gone through ldapssl_init -
          so, use NULL if using starttls */
       if (secure == 1) {
 		myld = ld;
       }
-
+#endif
 		/* verify certificate only */
 #if defined(USE_OPENLDAP)
       ssl_strength = LDAP_OPT_X_TLS_NEVER;
@@ -2413,55 +2462,13 @@ admutil_ldap_bind(
 )
 {
   int rc = LDAP_SUCCESS;
-  int secure = 0;
+  int err = LDAP_SUCCESS;
   struct berval bvcreds = {0, NULL};
   LDAPMessage *result = NULL;
   struct berval *servercredp = NULL;
-#if defined(USE_OPENLDAP)
-  /* openldap doesn't have a SSL/TLS yes/no flag - so grab the
-     ldapurl, parse it, and see if it is a secure one */
-  char *ldapurl = NULL;
 
-  ldap_get_option(ld, LDAP_OPT_URI, &ldapurl);
-  if (ldapurl && !PL_strncasecmp(ldapurl, "ldaps", 5)) {
-    secure = 1;
-  }
-  PL_strfree(ldapurl);
-  ldapurl = NULL;
-#else /* !USE_OPENLDAP */
-  ldap_get_option(ld, LDAP_OPT_SSL, &secure);
-#endif
-
-#ifdef EXTERNAL_AUTH_SUPPORTED
-  if (secure && mech && !strcmp(mech, LDAP_SASL_EXTERNAL)) {
-	/* SSL connections will use the server's security context
-	   and cert for client auth */
-	rc = slapd_SSL_client_auth(ld);
-
-	if (rc != 0) {
-#ifdef DEBUG
-      fprintf(stderr, "admutil_ldap_bind: "
-                      "Error: could not configure the server for cert "
-                      "auth - error %d - make sure the server is "
-                      "correctly configured for SSL/TLS\n", rc);
-#endif
-      goto done;
-	} else {
-#ifdef DEBUG
-      fprintf(stderr, "admutil_ldap_bind: "
-                      "Set up conn to use client auth\n");
-#endif
-    }
-	bvcreds.bv_val = NULL; /* ignore username and passed in creds */
-	bvcreds.bv_len = 0; /* for external auth */
-	bindid = NULL;
-  } else { /* other type of auth */
-#endif /* EXTERNAL_AUTH_SUPPORTED */
-	bvcreds.bv_val = (char *)creds;
-	bvcreds.bv_len = creds ? strlen(creds) : 0;
-#ifdef EXTERNAL_AUTH_SUPPORTED
-  }
-#endif /* EXTERNAL_AUTH_SUPPORTED */
+  bvcreds.bv_val = (char *)creds;
+  bvcreds.bv_len = creds ? strlen(creds) : 0;
 
   /* The connection has been set up - now do the actual bind, depending on
      the mechanism and arguments */
@@ -2519,10 +2526,21 @@ admutil_ldap_bind(
       }
       /* if we got here, we were able to read success result */
       /* Get the controls sent by the server if requested */
-      if (returnedctrls) {
-        if ((rc = ldap_parse_result(ld, result, &rc, NULL, NULL,
-                                    NULL, returnedctrls,
-                                    0)) != LDAP_SUCCESS) {
+      if ((rc = ldap_parse_result(ld, result, &err, NULL, NULL,
+                                  NULL, returnedctrls,
+                                  0)) != LDAP_SUCCESS) {
+#ifdef DEBUG
+          fprintf(stderr, "admutil_ldap_bind: "
+                          "Error: could not parse bind result:"
+                          " mech [%s]: error %d (%s)\n",
+                          mech ? mech : "SIMPLE",
+                          rc, ldap_err2string(rc));
+#endif
+          goto done;
+      }
+
+      if(err){
+          rc = err;
 #ifdef DEBUG
           fprintf(stderr, "admutil_ldap_bind: "
                           "Error: could not bind id "
@@ -2532,8 +2550,7 @@ admutil_ldap_bind(
                           rc, ldap_err2string(rc));
 #endif
           goto done;
-		}
-      }
+      } 
 
       /* parse the bind result and get the ldap error code */
       if ((rc = ldap_parse_sasl_bind_result(ld, result, &servercredp,
@@ -2577,6 +2594,260 @@ done:
   return rc;
 }
 
+#if defined(USE_OPENLDAP)
+#define LDAP_DN         1
+#define LDAP_RDN        2
+
+#define INQUOTE         1
+#define OUTQUOTE        2
+
+#define LDAP_UTF8LEN(s)  ((0x80 & *(unsigned char*)(s)) ?   ldap_utf8len (s) : 1)
+
+static char UTF8len[64]
+= {1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
+   1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
+   0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
+   2, 2, 2, 2, 2, 2, 2, 2, 3, 3, 3, 3, 4, 4, 5, 6};
+
+static char*
+ldap_utf8next (char* s)
+     /* Return a pointer to the character immediately following *s.
+        Handle any valid UTF-8 character, including '\0' and ASCII.
+        Try to handle a misaligned pointer or a malformed character.
+     */
+{
+    register unsigned char* next = (unsigned char*)s;
+    switch (UTF8len [(*next >> 2) & 0x3F]) {
+      case 0: /* erroneous: s points to the middle of a character. */
+      case 6: if ((*++next & 0xC0) != 0x80) break;
+      case 5: if ((*++next & 0xC0) != 0x80) break;
+      case 4: if ((*++next & 0xC0) != 0x80) break;
+      case 3: if ((*++next & 0xC0) != 0x80) break;
+      case 2: if ((*++next & 0xC0) != 0x80) break;
+      case 1: ++next;
+    }
+    return (char*) next;
+}
+
+static int
+ldap_utf8len (const char* s)
+     /* Return the number of char's in the character at *s. */
+{
+    return ldap_utf8next((char*)s) - s;
+}
+
+static int
+ldap_utf8isspace( char* s )
+{
+    register unsigned char *c = (unsigned char*)s;
+    int len = ldap_utf8len(s);
+
+    if (len == 0) {
+        return 0;
+    } else if (len == 1) {
+        switch (*c) {
+            case 0x09:
+            case 0x0A:
+            case 0x0B:
+            case 0x0C:
+            case 0x0D:
+            case 0x20:
+                return 1;
+            default:
+                return 0;
+        }
+    } else if (len == 2) {
+        if (*c == 0xc2) {
+                return *(c+1) == 0x80;
+        }
+    } else if (len == 3) {
+        if (*c == 0xE2) {
+            c++;
+            if (*c == 0x80) {
+                c++;
+                return (*c>=0x80 && *c<=0x8a);
+            }
+        } else if (*c == 0xE3) {
+            return (*(c+1)==0x80) && (*(c+2)==0x80);
+        } else if (*c==0xEF) {
+            return (*(c+1)==0xBB) && (*(c+2)==0xBF);
+        }
+        return 0;
+    }
+
+    /* should never reach here */
+    return 0;
+}
+
+static char **
+mozldap_ldap_explode( const char *dn, const int notypes, const int nametype )
+{
+        char    *p, *q, *rdnstart, **rdns = NULL;
+        size_t  plen = 0;
+        int             state = 0;
+        int             count = 0;
+        int             startquote = 0;
+        int             endquote = 0;
+        int             len = 0;
+        int             goteq = 0;
+
+        if ( dn == NULL ) {
+                dn = "";
+        }
+
+        while ( ldap_utf8isspace( (char *)dn )) { /* ignore leading spaces */
+                ++dn;
+        }
+
+        p = rdnstart = (char *) dn;
+        state = OUTQUOTE;
+
+        do {
+                p += plen;
+                plen = 1;
+                switch ( *p ) {
+                case '\\':
+                        if ( *++p == '\0' )
+                                p--;
+                        else
+                                plen = LDAP_UTF8LEN(p);
+                        break;
+                case '"':
+                        if ( state == INQUOTE )
+                                state = OUTQUOTE;
+                        else
+                                state = INQUOTE;
+                        break;
+                case '+': if ( nametype != LDAP_RDN ) break;
+                case ';':
+                case ',':
+                case '\0':
+                        if ( state == OUTQUOTE ) {
+                                /*
+                                 * semicolon and comma are not valid RDN
+                                 * separators.
+                                 */
+                                if ( nametype == LDAP_RDN &&
+                                        ( *p == ';' || *p == ',' || !goteq)) {
+                                        admutil_strsFree( rdns );
+                                        return NULL;
+                                }
+                                if ( (*p == ',' || *p == ';') && !goteq ) {
+                                   /* If we get here, we have a case similar
+                                    * to <attr>=<value>,<string>,<attr>=<value>
+                                    * This is not a valid dn */
+                                    admutil_strsFree( rdns );
+                                    return NULL;
+                                }
+                                goteq = 0;
+                                ++count;
+                                if ( rdns == NULL ) {
+                                        if (( rdns = (char **)PR_Malloc( 8
+                                                 * sizeof( char *))) == NULL )
+                                                return( NULL );
+                                } else if ( count >= 8 ) {
+                                        if (( rdns = (char **)PR_Realloc(
+                                            (char *)rdns, (count+1) *
+                                            sizeof( char *))) == NULL )
+                                                return( NULL );
+                                }
+                                rdns[ count ] = NULL;
+                                endquote = 0;
+                                if ( notypes ) {
+                                        for ( q = rdnstart;
+                                            q < p && *q != '='; ++q ) {
+                                                ;
+                                        }
+                                        if ( q < p ) { /* *q == '=' */
+                                                rdnstart = ++q;
+                                        }
+                                        if ( *rdnstart == '"' ) {
+                                                startquote = 1;
+                                                ++rdnstart;
+                                        }
+
+                                        if ( (*(p-1) == '"') && startquote ) {
+                                                endquote = 1;
+                                                --p;
+                                        }
+                                }
+
+                                len = p - rdnstart;
+                                if (( rdns[ count-1 ] = (char *)PR_Calloc(
+                                    1, len + 1 )) != NULL ) {
+                                        memcpy( rdns[ count-1 ], rdnstart,
+                                            len );
+                                        if ( !endquote ) {
+                                                /* trim trailing spaces */
+                                                while ( len > 0 &&
+                                                    ldap_utf8isspace(
+                                                    &rdns[count-1][len-1] )) {
+                                                        --len;
+                                                }
+                                        }
+                                        rdns[ count-1 ][ len ] = '\0';
+                                }
+
+                                /*
+                                 *  Don't forget to increment 'p' back to where
+                                 *  it should be.  If we don't, then we will
+                                 *  never get past an "end quote."
+                                 */
+                                if ( endquote == 1 )
+                                        p++;
+
+                                rdnstart = *p ? p + 1 : p;
+                                while ( ldap_utf8isspace( rdnstart ))
+                                        ++rdnstart;
+                        }
+                        break;
+                case '=':
+                        if ( state == OUTQUOTE ) {
+                                goteq = 1;
+                        }
+                        /* FALL */
+                default:
+                        plen = LDAP_UTF8LEN(p);
+                        break;
+                }
+        } while ( *p );
+
+        return( rdns );
+}
+
+static char **
+mozldap_ldap_explode_dn( const char *dn, const int notypes )
+{
+        return( mozldap_ldap_explode( dn, notypes, LDAP_DN ) );
+}
+
+static char **
+mozldap_ldap_explode_rdn( const char *rdn, const int notypes )
+{
+        return( mozldap_ldap_explode( rdn, notypes, LDAP_RDN ) );
+}
+#endif /* USE_OPENLDAP */
+
+char **
+admldap_ldap_explode_dn( const char *dn, const int notypes )
+{
+#if defined(USE_OPENLDAP)
+  return mozldap_ldap_explode_dn(dn, notypes);
+#else
+  return ldap_explode_dn(dn, notypes);
+#endif
+}
+
+char **
+admldap_ldap_explode_rdn( const char *rdn, const int notypes )
+{
+#if defined(USE_OPENLDAP)
+  return mozldap_ldap_explode_rdn(rdn, notypes);
+#else
+  return ldap_explode_rdn(rdn, notypes);
+#endif
+}
+
 /*
   emacs settings
   Local Variables:
