@@ -174,13 +174,13 @@ PR_IMPLEMENT(void)
 unescape_entities(char *s)
 {
     int		idx;
-    char	*p, *r, *d;
+    char	*p, *r;
 
     if (!s || !*s) {
         return;
     }
 
-    d = r = s;
+    r = s;
     for ( p = s; *p != '\0'; LDAP_UTF8INC( p )) {
         if ( ((*p) & 0x80) == 0 && ( (*p) == '&') ) {
             for( idx = 0; idx < entitynum; ++idx ) {
