@@ -51,19 +51,6 @@
 
 #define SECURITY_ATTR "configuration.nsServerSecurity"
 
-#ifdef XP_UNIX
-/*
- * Version information for the 'ident' and 'what commands
- *
- * NOTE: the first component of the concatenated rcsid string
- * must not end in a '$' to prevent rcs keyword substitution.
- */
-static char rcsid[] = "$Header: AdminUtil " ADMSDK_PRODUCT  
-        "  " ADMSDK_BUILDNUM " $";
-static char sccsid[] = "@(#)AdminUtil " ADMSDK_PRODUCT  
-		"  " ADMSDK_BUILDNUM;
-#endif /* XP_UNIX */
-
 void servssl_error(char *fmt, ...);
 
 char custom_file[BIG_LINE] = "";
@@ -106,6 +93,7 @@ admldapBuildInfoSSL(AdmldapInfo info, int *errorcode)
   } else {
       /* no password means just punt rather than do anon bind */
       /* this mimics the same logic in admldapBuildInfoCbk() */
+      ldap_unbind_ext(ld, NULL, NULL);
       *errorcode = ADMUTIL_LDAP_ERR;
       return 1; /* have to return true here to mimic admldapBuildInfoCbk() */
   }
@@ -115,10 +103,9 @@ admldapBuildInfoSSL(AdmldapInfo info, int *errorcode)
     case LDAP_INAPPROPRIATE_AUTH:
     case LDAP_INVALID_CREDENTIALS:
     case LDAP_INSUFFICIENT_ACCESS:
+    case LDAP_UNWILLING_TO_PERFORM:
       /* authenticate failed: Should not continue */
-      ldap_unbind_ext(ld, NULL, NULL);
-      *errorcode = ADMUTIL_LDAP_ERR;
-      return 0;
+      break;
     case LDAP_NO_SUCH_OBJECT:
     case LDAP_ALIAS_PROBLEM:
     case LDAP_INVALID_DN_SYNTAX:
